import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import sys

plt.style.use('dark_background')

# Load the CSV data
csv_file_path = sys.argv[1]
data = pd.read_csv(csv_file_path, delimiter=';')

# Filter the DataFrame for the specified PIDs
pids_of_interest = [
    'Vehicle speed',
    'HV EV Battery Power',
    'Vehicle acceleration',
    'Distance travelled',
    'Battery State of Charge',
    'Altitude (GPS)'
]

filtered_data = data[data['PID'].isin(pids_of_interest)].copy()

# Convert SECONDS to a readable datetime format with current date
start_time = pd.to_datetime('2024-09-01')
filtered_data['TIME'] = filtered_data['SECONDS'].apply(lambda x: start_time + pd.to_timedelta(x, unit='s'))

# Detect and remove noise based on a threshold for each variable
def remove_noise(df, noise_threshold):
    df = df.copy()  # Ensure we are working with a copy
    df['VALUE'] = pd.to_numeric(df['VALUE'], errors='coerce')
    df['DIFF'] = df['VALUE'].diff().abs()
    df = df[df['DIFF'] < noise_threshold].drop(columns=['DIFF'])
    return df

# Define thresholds for each PID
thresholds = {
    'Vehicle speed': 93,
    'HV EV Battery Power': 150,
    'Vehicle acceleration': 1,
    'Distance travelled': 999,
    'Battery State of Charge': 1,
    'Altitude (GPS)': 14000
}

# Define units for each PID
units = {
    'Vehicle speed': 'mph',
    'HV EV Battery Power': 'kW',
    'Vehicle acceleration': 'g',
    'Distance travelled': 'miles',
    'Battery State of Charge': '%',
    'Altitude (GPS)': 'feet'
}

# Define colors for each PID
colors = {
    'Vehicle speed': 'cyan',
    'HV EV Battery Power': 'magenta',
    'Vehicle acceleration': 'yellow',
    'Distance travelled': 'green',
    'Battery State of Charge': 'orange',
    'Altitude (GPS)': 'red'
}

# Define colors for each PID
colors = {
    'Vehicle speed': 'cyan',
    'HV EV Battery Power': 'magenta',
    'Vehicle acceleration': 'yellow',
    'Distance travelled': 'white',
    'Battery State of Charge': 'green',
    'Altitude (GPS)': 'brown'
}

# Apply noise removal
cleaned_data = pd.DataFrame()
for pid in pids_of_interest:
    pid_data = filtered_data[filtered_data['PID'] == pid].copy()
    threshold = thresholds[pid]
    cleaned_pid_data = remove_noise(pid_data, threshold)
    cleaned_data = pd.concat([cleaned_data, cleaned_pid_data])

# Plot the cleaned data for visual inspection
fig, axes = plt.subplots(len(pids_of_interest), figsize=(14, 12), sharex=True)

# Set the date format for the x-axis
date_format = mdates.DateFormatter('%I:%M %p')

for i, pid in enumerate(pids_of_interest):
    pid_data = cleaned_data[cleaned_data['PID'] == pid]
    axes[i].plot(pid_data['TIME'], pid_data['VALUE'])
    # Use the original TIME column for plotting
    axes[i].set_title(pid)
    axes[i].set_ylabel(f'{units[pid]}')
    axes[i].grid(True)
    axes[i].xaxis.set_major_formatter(date_format)
    # Apply the date for# Use the color for each PIDmat
    axes[i].plot(pid_data['TIME'], pid_data['VALUE'], color=colors[pid])
    # Darken the grid lines
    axes[i].grid(True, color='#444444')  # Dark gray grid lines

    # Darken the axes lines (spines)
    axes[i].spines['bottom'].set_color('#444444')
    axes[i].spines['top'].set_color('#444444')
    axes[i].spines['left'].set_color('#444444')
    axes[i].spines['right'].set_color('#444444')

    axes[i].xaxis.set_major_formatter(date_format)  # Apply the date format
    axes[i].set_facecolor('#121212')  # Darker background

axes[-1].set_xlabel('Time (MM/DD/YYYY HH:MM AM/PM)')
plt.setp(axes[-1].xaxis.get_majorticklabels(), rotation=45, ha="right")
plt.tight_layout()
plt.show()
